﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using KinematicCharacterController;
using KinematicCharacterController.Examples;
using UnityEngine.InputSystem;

namespace KinematicCharacterController.Examples
{
    public class ExamplePlayer : MonoBehaviour
    {
        public ExampleCharacterController Character;
        public ExampleCharacterCamera CharacterCamera;

        public PlayerInput playerInput;

        private PlayerCharacterInputs inputs = new PlayerCharacterInputs();

        private void Start()
        {
            // Tell camera to follow transform
            CharacterCamera.SetFollowTransform(Character.CameraFollowPoint);

            // Ignore the character's collider(s) for camera obstruction checks
            CharacterCamera.IgnoredColliders.Clear();
            CharacterCamera.IgnoredColliders.AddRange(Character.GetComponentsInChildren<Collider>());

        }

        private void Update()
        {
            HandleCharacterInput();
        }

        private void LateUpdate()
        {
            // Handle rotating the camera along with physics movers
            if (CharacterCamera.RotateWithPhysicsMover && Character.Motor.AttachedRigidbody != null)
            {
                CharacterCamera.PlanarDirection = Character.Motor.AttachedRigidbody.GetComponent<PhysicsMover>().RotationDeltaFromInterpolation * CharacterCamera.PlanarDirection;
                CharacterCamera.PlanarDirection = Vector3.ProjectOnPlane(CharacterCamera.PlanarDirection, Character.Motor.CharacterUp).normalized;
            }

            CharacterCamera.UpdateWithInput(Time.deltaTime, 0f, Vector2.zero);
        }

        private void HandleCharacterInput()
        {

            // Build the CharacterInputs struct
            inputs.CameraRotation = CharacterCamera.Transform.rotation;
            Character.SetInputs(ref inputs);
        }

        public void HandleMoveEvent(InputAction.CallbackContext context)
        {
            Vector2 moveValue = context.action.ReadValue<Vector2>();
            inputs.MoveAxisForward = moveValue.y;
            inputs.MoveAxisRight = moveValue.x;
        }

        public void HandleCrouchEvent(InputAction.CallbackContext context)
        {
            inputs.CrouchDown = context.ReadValueAsButton();
            inputs.CrouchUp = !inputs.CrouchDown; //Maybe problematic later haha
        }
        public void HandleLookEvent(InputAction.CallbackContext context)
        {
            // Create the look input vector for the camera
            Vector2 moveDirection = context.ReadValue<Vector2>();
            float mouseLookAxisUp = moveDirection.y;
            float mouseLookAxisRight = moveDirection.x;
            Vector3 lookInputVector = new Vector3(mouseLookAxisRight, mouseLookAxisUp, 0f);

            // Prevent moving the camera while the cursor isn't locked
            if (Cursor.lockState != CursorLockMode.Locked)
            {
                lookInputVector = Vector3.zero;
            }

            // Input for zooming the camera (disabled in WebGL because it can cause problems)
            float scrollInput = 0f;//-Input.GetAxis(MouseScrollInput);
#if UNITY_WEBGL
            scrollInput = 0f;
#endif

            // Apply inputs to the camera
            CharacterCamera.UpdateWithInput(Time.deltaTime, scrollInput, lookInputVector);

            // Handle toggling zoom level
/*            if (Input.GetMouseButtonDown(1))
            {
                CharacterCamera.TargetDistance = (CharacterCamera.TargetDistance == 0f) ? CharacterCamera.DefaultDistance : 0f;
            }*/
        }

        public void HandleClickEvent(InputAction.CallbackContext context)
        {
            if (context.action.name == "UnlockCamera")
            {
                Cursor.lockState = CursorLockMode.None;
            }
            else if (context.action.name == "Click")
            {
                Cursor.lockState = CursorLockMode.Locked;
            }
        }

        public void HandleJumpEvent(InputAction.CallbackContext context)
        {
            inputs.JumpDown = context.ReadValueAsButton();
        }
    }
}